######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/brw_hook.4gl  1.4  Delta: 3/9/91
######################################################################

globals "globals.4gl"

######################################################################
function brw_hook(ring_rowid, ring_cursor, ring_total)
# returning false, ring_rowid, ring_cursor, ring_total
######################################################################
#  if this function is called, that means that there is no local
#  brw_hook(), and don't link in the scrolling library.
#
# It is to be modified for other languages by copying the
# "menu to end menu" logic and adding more case statement values such as
# German "GRM", French "FRN", etc.
#
# The separate "modules" such as this allow for the various menus to 
# be accessed from a applications local directory for "menu" command
# modifications. These would include the deletions of menu_item commands 
# or the addition new menu-items.
#
# Add other language case values here and copy the menu through end
# menu logic here.  CHANGE THE COMMAND OPTIONS, THE PROMPT, AND
# ALL LABELS.
#
    define
        ring_rowid  integer,        # rowid of current selection
        ring_cursor integer,        # absolute cursor address
        ring_total  integer,
        menu_lang   char(3),        # language value loads into here
        tmpvar      char(40)

    # Trap fatal errors
    whenever error call error_handler

    # Get the language value
    let menu_lang = get_scrlib("language")

    case
    # Add other language case values here and copy the menu through end
    # menu logic here.  CHANGE THE COMMAND OPTIONS, THE PROMPT, AND
    # ALL LABELS.
    #
    # Example: when menu_lang = "GRM"

    # Bogus "when" to make "case" syntactically correct
    when menu_lang = "BOGUS"

    # Default value is english (ENG)
    otherwise
    menu " Browse"
        command "First  " " Move to first selected document"
            label m_first:
            call ring_view(1) returning ring_rowid, ring_cursor
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Last  " " Move to last selected document"
            label m_last:
            call ring_view(ring_total) returning ring_rowid, ring_cursor
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Next  " " Move to next document"
            label m_next:
            if ring_total = 0
            then
                continue menu
            end if
            if ring_cursor = ring_total
            then
                let ring_cursor = 0
            end if
            let ring_cursor = ring_cursor + 1
            call ring_view(ring_cursor) returning ring_rowid, ring_cursor
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Prev  " " Move to previous document"
            label m_prev:
            if ring_total = 0
            then
                continue menu
            end if
            if ring_cursor <= 1
            then
                let ring_cursor = ring_total + 1
            end if
            let ring_cursor = ring_cursor - 1
            call ring_view(ring_cursor)
              returning ring_rowid, ring_cursor
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command "Goto  " " Move to specified document"
            label m_goto:
            call ring_clear()
            let tmpvar = "Goto:  [1-",
              ring_total using "<,<<<,<<<" clipped,
              "]  or  [DEL] to Quit"
            display tmpvar at 1,2
            whenever error continue
            let int_flag = 0
            let status = 1
            while status != 0 and int_flag = 0
                let status = 0
                prompt " Move to the document in position: " for ring_rowid
            end while
            whenever error call error_handler
            if int_flag != 0
            then
                let int_flag = 0
                continue menu
            end if
            if ring_rowid >= 1 and ring_rowid <= ring_total
            then
                call ring_view(ring_rowid) returning ring_rowid, ring_cursor
            else
                if ring_rowid < 1
                then
                    call ring_view(1) returning ring_rowid, ring_cursor
                end if
                if ring_rowid > ring_total
                then
                    call ring_view(ring_total)
                      returning ring_rowid, ring_cursor
                end if
            end if
            call ring_border(ring_rowid, ring_cursor, ring_total)
        command key (e,x,q,"interrupt")
            "Quit" " Return to previous menu"
            label m_quit:
            exit menu
        command key ("!")
            call ring_bang()

      # Event trapping logic
      command key (control-b) let hotkey = 2  goto event
      command key (control-e) let hotkey = 5  goto event
      command key (control-f) let hotkey = 6  goto event
      command key (control-g) let hotkey = 7  goto event
      command key (control-i) let hotkey = 9  goto event
      command key (control-n) let hotkey = 14 goto event
      command key (control-o) let hotkey = 15 goto event
      command key (control-p) let hotkey = 16 goto event
      command key (control-t) let hotkey = 20 goto event
      command key (control-u) let hotkey = 21 goto event
      command key (control-v) let hotkey = 22 goto event
      command key (control-w) let hotkey = 23 goto event
      command key (control-y) let hotkey = 25 goto event
      command key (control-z) let hotkey = 26 goto event
      command key (f1)  let hotkey = 101 goto event
      command key (f2)  let hotkey = 102 goto event
      command key (f3)  let hotkey = 103 goto event
      command key (f4)  let hotkey = 104 goto event
      command key (f5)  let hotkey = 105 goto event
      command key (f6)  let hotkey = 106 goto event
      command key (f7)  let hotkey = 107 goto event
      command key (f8)  let hotkey = 108 goto event
      command key (f9)  let hotkey = 109 goto event
      command key (f10) let hotkey = 110 goto event
      command key (f11) let hotkey = 111 goto event
      command key (f12) let hotkey = 112 goto event
      command key (f13) let hotkey = 113 goto event
      command key (f14) let hotkey = 114 goto event
      command key (f15) let hotkey = 115 goto event
      command key (f16) let hotkey = 116 goto event
      command key (f34) let hotkey = 134 goto event
      command key (f35) let hotkey = 135 goto event
      command key (f36) let hotkey = 136 goto event

      label event:
        call hot_local("help")
        call hot_key("brw_hook")
        case
          when scr_funct = "m_first" goto m_first
          when scr_funct = "m_last" goto m_last
          when scr_funct = "m_next" goto m_next
          when scr_funct = "m_prev" goto m_prev
          when scr_funct = "m_goto" goto m_goto
          when scr_funct = "accept" exit menu
          when scr_funct = "cancel" goto m_quit
          when scr_funct = "help" call lib_help("lib_scr","brw_hook",1)
        end case

    end menu

    end case

    let scr_funct = ""
    # false return telling r_browse that the window browser wasn't used.
    return false, ring_rowid, ring_cursor, ring_total
end function
# brw_hook()
