######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/fg_prompt.4gl  1.1  Delta: 1/9/91
######################################################################

globals "globals.4gl"

######################################################################
function fg_prompt(line1, line2)
# returning true if the prompt was answered.
# returning false if interrupted or a local event occurs.
# scr_funct = "cancel" if interrupted (int_flag is reset to false)
# returning the answer in the scratch variable
######################################################################
# This function allows events to be processed during any prompt
# statement.
# If the user enters data, it is returned in the scratch variable,
# and the function returns true.
# If the user presses a hot key, and it results in a local event
# (like zoom, cancel, accept, etc.), the function returns a false
# and the event name is returned in scratch.
# Example: User presses [DEL] (or event key mapped to [DEL]):
#  fg_prompt() returns a false, and "cancel" in scratch.
# This will allow you to add local events like "zoom", "accept", etc.
# Normally, fg_prompt is used as follows:
#   if fg_prompt("Command:  Description", "") = false
#     then return  # exit function if user didn't enter any data
#   end if
# A more advanced need may require a zoom:
#   if fg_prompt("Command:  Description", "(or [ZOOM])") = false
#   then
#       if scratch = "zoom"
#         then call zoom("screen","") end if  # zoom pressed
#         else return                         # DEL or other event
#       end if
#   end if
#
    define
        line1 char(74),      # First line
        line2 char(60),      # Prompt line
        prompt_cont char(1), # Continue the prompting?
        n smallint           # Generic number

    # Trap fatal errors
    whenever error call error_handler

    # Setup for the prompt loop
    let scr_funct = ""
    let prompt_cont = "Y"
    let int_flag = false
    while prompt_cont = "Y"
        let prompt_cont = "N"
        if line1 is not null then display line1 at 1,2 end if

        # Check for auto answer
        if get_scrlib("auto_answr") = "Y" then exit while end if

        # Prompt
        prompt " ", line2 clipped, ": " for scratch

          # Event trapping logic
          on key (control-b) let hotkey = 2  goto event
          on key (control-e) let hotkey = 5  goto event
          on key (control-f) let hotkey = 6  goto event
          on key (control-g) let hotkey = 7  goto event
          on key (control-i) let hotkey = 9  goto event
          on key (control-n) let hotkey = 14 goto event
          on key (control-o) let hotkey = 15 goto event
          on key (control-p) let hotkey = 16 goto event
          on key (control-t) let hotkey = 20 goto event
          on key (control-u) let hotkey = 21 goto event
          on key (control-v) let hotkey = 22 goto event
          on key (control-w) let hotkey = 23 goto event
          on key (control-y) let hotkey = 25 goto event
          on key (control-z) let hotkey = 26 goto event
          on key (f1)  let hotkey = 101 goto event
          on key (f2)  let hotkey = 102 goto event
          on key (f3)  let hotkey = 103 goto event
          on key (f4)  let hotkey = 104 goto event
          on key (f5)  let hotkey = 105 goto event
          on key (f6)  let hotkey = 106 goto event
          on key (f7)  let hotkey = 107 goto event
          on key (f8)  let hotkey = 108 goto event
          on key (f9)  let hotkey = 109 goto event
          on key (f10) let hotkey = 110 goto event
          on key (f11) let hotkey = 111 goto event
          on key (f12) let hotkey = 112 goto event
          on key (f13) let hotkey = 113 goto event
          on key (f14) let hotkey = 114 goto event
          on key (f15) let hotkey = 115 goto event
          on key (f16) let hotkey = 116 goto event
          on key (f34) let hotkey = 134 goto event
          on key (f35) let hotkey = 135 goto event
          on key (f36) let hotkey = 136 goto event

          # Local event processing
          label event:
            call hot_key("fg_prompt")  # Map the key to the event
            if scr_funct is null
              then let prompt_cont = "Y" # Non-local event: Prompt again
            end if

        end prompt
    end while

    # Interrupt pressed
    if int_flag != 0
    then
        let scr_funct = "cancel"
        let int_flag = 0
    end if

    # Scr_funct null means no local event was selected
    if scr_funct is null
      then return true
      else return false
    end if
end function
# fg_prompt()
