######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/l_b4aftr.4gl  1.5  Delta: 1/2/91
######################################################################

globals "globals.4gl"

define
    is_zoom smallint     # (boolean) are we in a zoom type field?

######################################################################
function lib_before(fld_name)
######################################################################
# This function is called before every screen field.  It is passed
# the screen field name in table.column format.
# It does the following:
# - returns if nxt_fld has something in it
# - sets p_cur and s_cur globals (only used in array type inputs)
# - sets the "(zoom)" message on the screen if scratch = "zoom"
# - sets the prev_data variable if first entry into this field
# - sets the scr_fld global variable
# - redisplays the data to clear any prior highlights
# - highlights the cursor on this field
#
    define
        fld_name char(40)

    # Trap fatal errors
    whenever error call error_handler

    # This must be retained here for backward compatibility.
    # (for those input statements that don't set
    # p_cur and s_cur in their 'before row' processing)
    let p_cur = arr_curr()
    let s_cur = scr_line()

    # If nxt_fld has something in it, then this field will be
    # skipped.  No use doing any before field logic on it.
    if nxt_fld is not null then return end if

    # Set zoom flag (if passed in scratch)
    if scratch = "zoom"
    then
        let is_zoom = true
        call lib_message("zoom_on")
        let scratch = ""
    end if

    # Set the scr_fld and prev_data variables
    # (when first entering this field)
    if scr_fld = fld_name
    then else
        let scr_fld = fld_name
        call switchbox("set this_data")
        let prev_data = this_data
        let data_changed = false
    end if

    # Re display the data to remove any prior highlights
    # and to change the color of the data.
    call switchbox("showdata")

    # Highlight the new data entry field
    call switchbox("highlight")

end function
# lib_before()

######################################################################
function lib_after()
######################################################################
# This function is called after every screen field.
# It does the following:
# - sets the global this_data variable (for comparison w/prev_data)
# - sets data_changed to true if prev_data is different than this_data
# - resets the "(zoom)" message on the screen (if set)
# - un-highlights the cursor on this field
#

    # Set the this_data variable
    call switchbox("set this_data")

    # Set the changed flag if data has changed
    if this_data = prev_data or
       (this_data is null and prev_data is null)
    then else
        let data_changed = true
    end if

    # Reset the zoom message
    if is_zoom
    then
        let is_zoom = false
        call lib_message("zoom_off")
    end if

    # This might be the last chance to display any partially
    # entered fields before exiting input, so re-display all
    # fields.  Only data in the screen that has been changed
    # will be redrawn.
    call switchbox("showdata")

end function
# lib_after()


######################################################################
function set_fldtab(scrn_fld)
# returning the column name portion of scrn_fld
######################################################################
# This function is designed to take the tablename portion of the
# scrn_fld and put it into the scrlib, and return only the
# column name of scrn_fld.  scrn_fld upon calling should look like:
# "tablename.columnname", and the returned field will look like
# "columnname" only.
#
    define
        scrn_fld char(40),  # Field in tablename.columnname format
        scr_len smallint,   # Length of scrn_fld
        n smallint          # Generic

    # Initialize some variables
    let scratch = " "
    let scr_len = length(scrn_fld)
    if scr_len = 0 then return " " end if

    # Process scr_fld until "." character
    for n = 1 to scr_len
        # Put chars of scrn_fld into scratch until we reach a "."
        if scrn_fld[n] = "."
          then exit for
        end if
        let scratch[n] = scrn_fld[n]
    end for

    # If scrn_fld = scratch, then it doesn't contain a "."
    # (not in "tablename.columnname" format)
    if scrn_fld = scratch
    then
        let scratch = " "
        let n = 0
    else
        # Set the tablename portion in scrlib
        call put_scrlib("fld_tab","(see scratch)")
    end if

    # Return the columnname portion
    return scrn_fld[n + 1, scr_len]

end function
# set_fldtab()
