######################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/menudetl.4gl  1.3  Delta: 1/4/91
######################################################################

globals "globals.4gl"

######################################################################
function detl_menu(ring_rowid,ring_cursor,ring_total)
######################################################################
# This is the main menu interface for header/detail screens.
# This is called from ring_detail and passed the "ring_" values
# to do the basic menuing logic. It is to be modified for other
# languages by copying the "menu to end menu" logic and adding
# more case statement values such as German "GRM", French "FRN", etc.
#
# The separate "modules" such as this allow for the various menus to
# be accessed from a applications local directory for "menu" command
# modifications. These would include the deletions of menu_item
# commands or the addition new menu-items.
#
# Add other language case values here and copy the menu through end
# menu logic here.  CHANGE THE COMMAND OPTIONS, THE PROMPT, AND
# ALL LABELS.
#

    define
        ring_rowid  integer, # the rowid of the current screen record
                             # or -1 if there is no record on the screen
        ring_cursor integer, # pointer to the absolute cursor address.
                             # undefined for a closed cursor
        ring_total  integer, # total number of elements in the cursor.
                             # if ring_total > 0, then cursor is open.
                             # if ring_total = 0, then cursor is closed.

        menu_lang   char(3)  # language value loads into here

    # If ring_cursor is 0 and ring_total > 0, then there is a open
    # cursor, but the record on the screen doesn't have anything
    # to do with it.  (a newly added document)

    # Trap fatal errors
    whenever error call error_handler

    # Get the language value
    let menu_lang = get_scrlib("language")

    # The MENU statements
    case

    # Add other language case values here and copy the menu through end
    # menu logic here.  CHANGE THE COMMAND OPTIONS, THE PROMPT, AND
    # ALL LABELS.
    #
    # Example: when menu_lang = "GRM"

    # Bogus "when" to make "case" syntactically correct
    when menu_lang = "BOGUS"

    # Default value is english (ENG)
    otherwise
    menu " Action"
      command "Add" " Create a new document"
        label m_add:
        let menu_item = "add"
        call ring_add(2,ring_rowid, ring_cursor, ring_total)
          returning ring_rowid, ring_cursor, ring_total
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Update" " Change this document"
        label m_update:
        let menu_item = "update"
        call ring_refresh(ring_rowid) returning ring_rowid
        call ring_update(2,ring_rowid)
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Delete" " Erase this document"
        label m_delete:
        let menu_item = "delete"
        call ring_delete(ring_rowid) returning ring_rowid
        if ring_rowid = -1 and ring_total > 0
        then
            let ring_rowid = 0      # deleted cursor element
            call put_scrlib("curs_rowid",0)
        end if
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Find" " Select and/or Reorder a group of documents"
        label m_find:
        let menu_item = "find"
        call ring_find(ring_rowid,ring_cursor,ring_total)
          returning ring_rowid, ring_cursor, ring_total
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Browse" " Page through selected documents"
        label m_browse:
        let menu_item = "browse"
        call ring_browse(ring_rowid,ring_cursor,ring_total)
          returning ring_rowid, ring_cursor, ring_total
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Nxt" " View next document"
        label m_next:
        let menu_item = "next"
        if ring_total = 0 then continue menu end if
        if ok_next() = false then continue menu end if
        if ring_cursor = ring_total then let ring_cursor = 0 end if
        let ring_cursor = ring_cursor + 1
        call put_scrlib("curs_pos",ring_cursor)
        call ring_view(ring_cursor) returning ring_rowid, ring_cursor
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Prv" " View previous document"
        label m_prev:
        let menu_item = "prev"
        if ring_total = 0 then continue menu end if
        if ok_prev() = false then continue menu end if
        if ring_cursor <= 1 then let ring_cursor = ring_total + 1 end if
        let ring_cursor = ring_cursor - 1
        call put_scrlib("curs_pos",ring_cursor)
        call ring_view(ring_cursor) returning ring_rowid, ring_cursor
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Tab" " Access document detail lines"
        label m_tab:
        let menu_item = "tab"
        if ok_tab() = false then continue menu end if
        call lib_message("scroll")
        call mld_scroll()
        let int_flag = 0
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command "Options" " Additional options"
        label m_options:
        let menu_item = "options"
        call ring_options(ring_rowid,ring_cursor,ring_total)
          returning ring_rowid, ring_cursor, ring_total
        call ring_border(ring_rowid, ring_cursor, ring_total)
      command key (e,x,q) "Quit" " End the program"
        label m_quit:
        let menu_item = "quit"
        if ok_exit() then exit menu end if
      command key ("!")
        call ring_bang()

      # Event trapping logic
      command key (control-b) let hotkey = 2  goto event
      command key (control-e) let hotkey = 5  goto event
      command key (control-f) let hotkey = 6  goto event
      command key (control-g) let hotkey = 7  goto event
      command key (control-i) let hotkey = 9  goto event
      command key (control-n) let hotkey = 14 goto event
      command key (control-o) let hotkey = 15 goto event
      command key (control-p) let hotkey = 16 goto event
      command key (control-t) let hotkey = 20 goto event
      command key (control-u) let hotkey = 21 goto event
      command key (control-v) let hotkey = 22 goto event
      command key (control-w) let hotkey = 23 goto event
      command key (control-y) let hotkey = 25 goto event
      command key (control-z) let hotkey = 26 goto event
      command key (f1)  let hotkey = 101 goto event
      command key (f2)  let hotkey = 102 goto event
      command key (f3)  let hotkey = 103 goto event
      command key (f4)  let hotkey = 104 goto event
      command key (f5)  let hotkey = 105 goto event
      command key (f6)  let hotkey = 106 goto event
      command key (f7)  let hotkey = 107 goto event
      command key (f8)  let hotkey = 108 goto event
      command key (f9)  let hotkey = 109 goto event
      command key (f10) let hotkey = 110 goto event
      command key (f11) let hotkey = 111 goto event
      command key (f12) let hotkey = 112 goto event
      command key (f13) let hotkey = 113 goto event
      command key (f14) let hotkey = 114 goto event
      command key (f15) let hotkey = 115 goto event
      command key (f16) let hotkey = 116 goto event
      command key (f34) let hotkey = 134 goto event
      command key (f35) let hotkey = 135 goto event
      command key (f36) let hotkey = 136 goto event

      label event:
        let menu_item = "menu"
        call hot_local("help")
        call hot_key("ring_detail")
        case
          when scr_funct = "m_add" goto m_add
          when scr_funct = "m_update" goto m_update
          when scr_funct = "m_delete" goto m_delete
          when scr_funct = "m_find" goto m_find
          when scr_funct = "m_browse" goto m_browse
          when scr_funct = "m_options" goto m_options
          when scr_funct = "m_next" goto m_next
          when scr_funct = "m_prev" goto m_prev
          when scr_funct = "m_tab" goto m_tab
          when scr_funct = "m_quit" goto m_quit
          when scr_funct = "tab" or scr_funct = "btab" goto m_tab
          when scr_funct = "accept" exit menu
          when scr_funct = "help" call fg_help(1)
        end case

    end menu

    end case

    return
end function
# detl_menu()
