########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/menuscrl.4gl  1.6  Delta: 3/23/91
########################################################################

globals "globals.4gl"

define
    arr_mesgs array[4] of record    # Message text
        mssg_text char(132)
    end record,
    mssg_prep char(1)               # Y/null Message cursor prepared?

######################################################################
function scrl_menu(ring_rowid, ring_cursor, ring_total,curr_line,
                    num_rows)
# returning ring_rowid, ring_cursor, ring_total
######################################################################
# This is the function that is called when "browse screen" is selected
# from the menu.  it is passed the "ring_" variables along with line
# and row and message variables.  It returns the "ring_" variables to
# the calling function "ring_scroll".
#
# It is to be modified for other languages by copying the
# "menu to end menu" logic and adding more case statement values such as
# German "GRM", French "FRN", etc.
#
# The separate "modules" such as this allow for the various menus to
# be accessed from a applications local directory for "menu" command
# modifications. These would include the deletions of menu_item
# commands or the addition new menu-items.
#
# Add other language case values here and copy the menu through end
# menu logic here.  CHANGE THE COMMAND OPTIONS, THE PROMPT, AND
# ALL LABELS.
#
# Messages:
#1: " There are no more rows in the direction you are going "
#2: "Goto:  [1-"
#3: "]  or  [DEL] to Quit"
#4: " Move to the document in position: " for ring_rowid
#
    define
        first_rowid integer,    # the initial row_id upon entering browse
        first_cursor integer,   # the initial cursor position
        first_total  integer,   # the initial number of rows in cursor
        ring_rowid  integer,    # the rowid of the current document
                                # (or -1 if no document is displayed)
        ring_cursor integer,    # the absolute address of the cursor
                                # (0 if document isn't part of the
                                # current cursor group)
        ring_total  integer,    # total number of elements in the cursor
        num_rows    smallint,   # number of rows in array
        curr_line   smallint,   # current line number
        tempstr     char(40),   # scratch string
        menu_lang   char(3)     # language value loads into here

    # Trap fatal errors
    whenever error call error_handler

   # set cursor information upon browse entry
   let first_rowid = ring_rowid
   let first_cursor = ring_cursor
   let first_total = ring_total
   let quit_flag = 0

    # Initialize messages
    if mssg_prep is null then
       call fg_getmsg("lib_scr","menuscrl",1)
       let arr_mesgs[1].mssg_text = scratch
       call fg_getmsg("lib_scr","menuscrl",2)
       let arr_mesgs[2].mssg_text = scratch
       call fg_getmsg("lib_scr","menuscrl",3)
       let arr_mesgs[3].mssg_text = scratch
       call fg_getmsg("lib_scr","menuscrl",4)
       let arr_mesgs[4].mssg_text = scratch
       let mssg_prep = "Y"
    end if

    # Get the language value
    let menu_lang = get_scrlib("language")

    case
    # Add other language case values here and copy the menu through end
    # menu logic here.  CHANGE THE COMMAND OPTIONS, THE PROMPT, AND
    # ALL LABELS.
    #
    # Example: when menu_lang = "GRM"

    # Bogus "when" to make "case" syntactically correct
    when menu_lang = "BOGUS"

    # Default value is english (ENG)
    otherwise
    menu " Browse"
        command "Next" " Move to next document"
            label m_next:
            if ring_cursor = ring_total
            then
                error arr_mesgs[1].mssg_text clipped
                continue menu
            end if

            # unlight current highligted item
            call brw_display(curr_line,false)

            let ring_cursor = ring_cursor + 1
            if curr_line = num_rows
            then # page down
                let curr_line = 1
                call ring_draw_win(num_rows, ring_cursor, ring_total)
            else
                let curr_line = curr_line + 1
            end if
            call ring_highlight(curr_line, ring_cursor)
                returning ring_rowid
            call ring_brw_border(ring_rowid, ring_cursor, ring_total)
        command "Prev" " Move to previous document"
            label m_prev:
            if ring_cursor = 1
            then
                error arr_mesgs[1].mssg_text clipped
                continue menu
            end if

            # unlight current highligted item
            call brw_display(curr_line,false)

            let ring_cursor = ring_cursor - 1
            if curr_line = 1
            then # page up
                if (ring_cursor - num_rows + 1) <= 1
                then # go to top
                    let curr_line = ring_cursor
                    let ring_cursor = 1
                    call ring_draw_win(num_rows, ring_cursor, ring_total)
                    let ring_cursor = curr_line
                else
                    let curr_line = num_rows
                    let ring_cursor = ring_cursor - num_rows + 1
                    call ring_draw_win(num_rows, ring_cursor, ring_total)
                    let ring_cursor = ring_cursor + num_rows - 1
                end if
            else
                let curr_line = curr_line - 1
            end if
            call ring_highlight(curr_line, ring_cursor)
                returning ring_rowid
            call ring_brw_border(ring_rowid, ring_cursor, ring_total)
        command "Up" " Move up one page"
            label m_pgup:
            if ring_cursor = 1
            then
                error arr_mesgs[1].mssg_text clipped
                continue menu
            end if

            # unlight current highligted item
            call brw_display(curr_line,false)

            if (ring_cursor - num_rows - curr_line + 1) < 1
            then # go to top
                let curr_line = 1
                let ring_cursor = 1
                call ring_draw_win(num_rows, ring_cursor, ring_total)
            else
                let ring_cursor = ring_cursor - num_rows - curr_line + 1
                call ring_draw_win(num_rows, ring_cursor, ring_total)
                let ring_cursor = ring_cursor + curr_line - 1
            end if
            call ring_highlight(curr_line, ring_cursor)
                returning ring_rowid
            call ring_brw_border(ring_rowid, ring_cursor, ring_total)
        command "Down" " Move down one page"
            label m_pgdn:
            if ring_cursor = ring_total
            then
                error arr_mesgs[1].mssg_text clipped
                continue menu
            end if

            # unlight current highligted item
            call brw_display(curr_line,false)

            if (ring_cursor + num_rows) > ring_total
            then # go to bottom
                if num_rows <= ring_total
                then
                    let curr_line = num_rows
                    let ring_cursor = ring_total - num_rows + 1
                    call ring_draw_win(num_rows, ring_cursor, ring_total)
                    let ring_cursor = ring_total
                else
                    let curr_line = ring_total
                    let ring_cursor = 1
                    call ring_draw_win(num_rows, ring_cursor, ring_total)
                    let ring_cursor = ring_total
                end if
            else # just page down
                let ring_cursor = ring_cursor + num_rows - curr_line + 1
                call ring_draw_win(num_rows, ring_cursor, ring_total)
                let ring_cursor = ring_cursor + curr_line - 1
            end if
            call ring_highlight(curr_line, ring_cursor)
                returning ring_rowid
            call ring_brw_border(ring_rowid, ring_cursor, ring_total)
        command "Top" " Move to first selected document"
            label m_first:
            let curr_line = 1
            let ring_cursor = 1
            call ring_draw_win(num_rows, ring_cursor, ring_total)
            call ring_highlight(curr_line, ring_cursor)
                returning ring_rowid
            call ring_brw_border(ring_rowid, ring_cursor, ring_total)
        command "Bottom" " Move to last selected document"
            label m_last:
            if num_rows <= ring_total
            then
                let curr_line = num_rows
                let ring_cursor = ring_total - num_rows + 1
                call ring_draw_win(num_rows, ring_cursor, ring_total)
                let ring_cursor = ring_total
            else
                let curr_line = ring_total
                let ring_cursor = 1
                call ring_draw_win(num_rows, ring_cursor, ring_total)
                let ring_cursor = ring_total
            end if
            call ring_highlight(curr_line, ring_cursor)
                returning ring_rowid
            call ring_brw_border(ring_rowid, ring_cursor, ring_total)
        command key (s,"escape") "Select" " Choose this document"
            label m_select:
            exit menu
        command "Goto" " Move to specified document"
            label m_goto:
            call ring_clear()
            let tempstr = arr_mesgs[2].mssg_text clipped,
              ring_total using "<,<<<,<<<" clipped,
               arr_mesgs[3].mssg_text clipped
            display tempstr at 1,2
            whenever error continue
            let int_flag = 0
            let status = 1
            while status != 0 and int_flag = 0
                let status = 0
                prompt arr_mesgs[4].mssg_text clipped for ring_rowid
            end while
            whenever error call error_handler
            if int_flag != 0
            then
                let int_flag = 0
                continue menu
            end if
            if ring_rowid >= 1 and ring_rowid <= ring_total
            then
                let ring_cursor = ring_rowid
            else
                if ring_rowid < 1
                then
                    let ring_cursor = 1
                end if
                if ring_rowid > ring_total
                then
                    let ring_cursor = ring_total
                end if
            end if
            let curr_line = 1
            call ring_draw_win(num_rows, ring_cursor, ring_total)
            call ring_highlight(curr_line, ring_cursor)
                returning ring_rowid
            call ring_brw_border(ring_rowid, ring_cursor, ring_total)
        command key (q,e,x,"interrupt")
            "Quit" " Return without selecting a document"
            label m_quit:
            let quit_flag = 1
            exit menu
        command key ("!")
            call ring_bang()

      # Event trapping logic
      command key (control-b) let hotkey = 2  goto event
      command key (control-e) let hotkey = 5  goto event
      command key (control-f) let hotkey = 6  goto event
      command key (control-g) let hotkey = 7  goto event
      command key (control-i) let hotkey = 9  goto event
      command key (control-n) let hotkey = 14 goto event
      command key (control-o) let hotkey = 15 goto event
      command key (control-p) let hotkey = 16 goto event
      command key (control-t) let hotkey = 20 goto event
      command key (control-u) let hotkey = 21 goto event
      command key (control-v) let hotkey = 22 goto event
      command key (control-w) let hotkey = 23 goto event
      command key (control-y) let hotkey = 25 goto event
      command key (control-z) let hotkey = 26 goto event
      command key (f1)  let hotkey = 101 goto event
      command key (f2)  let hotkey = 102 goto event
      command key (f3)  let hotkey = 103 goto event
      command key (f4)  let hotkey = 104 goto event
      command key (f5)  let hotkey = 105 goto event
      command key (f6)  let hotkey = 106 goto event
      command key (f7)  let hotkey = 107 goto event
      command key (f8)  let hotkey = 108 goto event
      command key (f9)  let hotkey = 109 goto event
      command key (f10) let hotkey = 110 goto event
      command key (f11) let hotkey = 111 goto event
      command key (f12) let hotkey = 112 goto event
      command key (f13) let hotkey = 113 goto event
      command key (f14) let hotkey = 114 goto event
      command key (f15) let hotkey = 115 goto event
      command key (f16) let hotkey = 116 goto event
      command key (f34) let hotkey = 134 goto event
      command key (f35) let hotkey = 135 goto event
      command key (f36) let hotkey = 136 goto event

      label event:
        call hot_local("help")
        call hot_key("ring_scroll")
        case
          when scr_funct = "m_first" goto m_first
          when scr_funct = "m_last" goto m_last
          when scr_funct = "m_next" goto m_next
          when scr_funct = "m_prev" goto m_prev
          when scr_funct = "m_goto" goto m_goto
          when scr_funct = "m_select" goto m_select
          when scr_funct = "m_pgup" goto m_pgup
          when scr_funct = "m_pgdn" goto m_pgdn
          when scr_funct = "accept" goto m_select
          when scr_funct = "cancel" goto m_quit
          when scr_funct = "help" call lib_help("lib_scr","browse",1)
        end case

    end menu

    end case

  # if quit or interrupt then reset to original cursor position
    if quit_flag = 1 then
    let ring_rowid = first_rowid
    let ring_cursor = first_cursor
    let ring_total = first_total
    end if

    return ring_rowid, ring_cursor, ring_total

end function
# scrl_menu()
