########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/mlh_curs.4gl  1.4  Delta: 3/23/91
########################################################################

# mlh_curs.4gl
# This file contains all of the logic for the FourGen browse cursor
# implimentation.
# The calls to these functions are designed to replace the local
# mlh_define/fetch/close_cur functions.

globals "globals.4gl"

define
    is_open char(1),    # (boolean) Is the browse table open?
    arr_mesgs array[3] of record    # Message text
        mssg_text char(132)
    end record,
    mssg_prep char(1)                  # Y/null Message cursor prepared?


######################################################################
function mlh_define_cur(criteria)
######################################################################
# This function is called when a browse cursor needs to be defined.
# If it is present in local code (not library), then it uses the old
# method of cursor manipulation.  If it isn't present in local code
# (codegenerator version 3.0+), then this function is loaded & the
# FourGen temp table method is used.
#
# Display Messages:
#1: "Sort:"
#2: " Do you want to re-define the order? (y/n) "
#3: "(Selecting: "
#
    define
        criteria char(512),    # user defined portion of filter
        tab_name char(18),     # browse table name
        hard_filter char(512), # hardcoded filter
        sel_column char(10),   # seq_no or rowid (based on turbo or SE)
        row_id integer,        # current rowid being processed
        seq_no integer,        # row sequence number
        c char(1),             # entry character
        m integer,             # generic counter
        n smallint             # generic counter

    # Trap fatal errors
    whenever error call error_handler

    # Initialize the display messages
    if mssg_prep is null then
       call fg_getmsg("lib_scr","mlhcur",1)
       let arr_mesgs[1].mssg_text = scratch
       call fg_getmsg("lib_scr","mlhcur",2)
       let arr_mesgs[2].mssg_text = scratch
       call fg_getmsg("lib_scr","mlhcur",3)
       let arr_mesgs[3].mssg_text = scratch
       let mssg_prep = "Y"
    end if

    # Set to no rows found (if early exit)
    let sqlca.sqlerrd[3] = 0

#   # Ask the user if they want another sort
#   if menu_item = "find"
#   then
#       call ring_clear()
#       display arr_mesgs[1].mssg_text clipped at 1,2
#       prompt arr_mesgs[2].mssg_text clipped  for char c
#       if int_flag
#       then
#           let int_flag = false
#           return
#       end if
#       if downshift(c) = "y"
#       then
#           call lib_message("sort")
#           let scratch = mlh_construct()
#           let sql_order = lib_order()
#       end if
#       call ring_clear()
#       call please_wait()
#   end if

    # Determine the tables, filter, and order
    let sql_filter = criteria
    call mlh_cursor()
    let tab_name = get_vararg()
    let hard_filter = get_vararg()
    if sql_order is null or sql_order = " "
      then let sql_order = get_vararg()
    end if

    # Default hard_filter to "1=1" if appropriate
    if hard_filter = " " or hard_filter is null
      then let hard_filter = "1=1" end if

    # Set the default sql_filter and sql_order
    call lib_filtord()

    # Build the SQL string
    let scratch = "select rowid"

    # Add the ordering columns
    let n = length(sql_order)
    if n > 0
    then
        # We have to remove any 'desc' portion of the order by string.
        let scratch = scratch clipped, ","
        let n = length(scratch) + 2
        for m = 11 to length(sql_order)
            # Bump up 'm' if 'desc' is found.
            if sql_order[m,m+4] = " desc"
            then
                # Don't get fooled by " description" or " desc_a"
                if  (sql_order[m+5] >= "a" and sql_order[m+5] <= "z")
                 or (sql_order[m+5] =  "_")
                  then  # ' desc' is part of a column (or table) name
                  else let m = m + 5  # skip over the ' desc' keyword
                end if
            end if
            # Transfer the character from sql_order into scratch
            let scratch[n] = sql_order[m]
            let n = n + 1
        end for
    end if

    # Continue with the 'from' and 'where' portions of the SQL string.
    let scratch = scratch clipped, " from ", tab_name clipped,
      " where ", hard_filter clipped, " and ", sql_filter clipped,
      sql_order clipped

    # Convert the string into an SQL cursor
    prepare brw_specs from scratch
    declare brw_cursor cursor for brw_specs
    open brw_cursor

    # Drop the old temp table
    if is_open is not null then drop table browse end if

    # Build the temp table
    let scratch =
      "create temp table browse(row_id integer, seq_no integer)"

    # Don't log the temp table if using transactions
    # (this works on 4.0 engines and above only)
    if is_trx() and is_4_0()
        then let scratch = scratch clipped, " with no log"
    end if
    prepare bld_tmp from scratch
    execute bld_tmp
    let is_open = "Y"

    # Build cursors on the temp table:

    # Declare an insert cursor for maximum efficiency
    let scratch = "insert into browse values(?, ?)"
    prepare ins_prep from scratch
    declare ins_cursor cursor for ins_prep

    # Determine the column to select on based on turbo running
    if is_turbo()
      then let sel_column = "seq_no"
      else let sel_column = "rowid"
    end if
    let scratch = "select row_id from browse where ",
      sel_column, " = ?"
    prepare fetch_stmt from scratch
    declare fetch_cur cursor for fetch_stmt

    open ins_cursor

    # Here we go:  For each element returned, insert the
    # data into the temp table.

    let seq_no = 0
    foreach brw_cursor into row_id
        if int_flag != 0 then exit foreach end if
        let seq_no = seq_no + 1
        let m = seq_no / 100
        if (m * 100) = seq_no
        then
            let scratch = arr_mesgs[3].mssg_text clipped,
                          seq_no using "<,<<<,<<<", ")"
            # Don't clear bottom line on successive calls
            if m > 100
              then let scratch[100,104] = "no cl"
            end if
            call lib_message("scr_bottom")
        end if
        put ins_cursor from row_id, seq_no
    end foreach

    # flush the cursor
    close ins_cursor
    close brw_cursor

    # Return the number of elements in the cursor
    let int_flag = 0
    let sqlca.sqlerrd[3] = seq_no

    return
end function
# mlh_define_cur()

######################################################################
function mlh_fetch_cur(curno)
# returning rowid in sqlca.sqlerrd[6]
######################################################################
#
    define
        curno integer,    # absolute cursor number
        row_id integer    # rowid to fetch

    open fetch_cur using curno
    fetch fetch_cur into row_id
    close fetch_cur
    let sqlca.sqlerrd[6] = row_id
end function
# mlh_fetch_cur()

######################################################################
function mlh_close_cur()
######################################################################
#
    return
end function
# mlh_close_cur()
