######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/scr_help.4gl  1.2  Delta: 12/18/90
######################################################################

globals "globals.4gl"

######################################################################
function scr_help()
######################################################################
# This is the context sensitive screen help function.
# IF we're in a screen field (scr_fld contains something)
# then show help for the field:
#   hlp_mod: field's tablename
#            (unless the field's tablename is "formonly".
#             in that case, the module = "module.program" for default
#             screens, and "module.scrid" for non-default screens)
#  hlp_prog: scr_fld - the field we're currently in
#   hlp_num: 1 (unless formonly, then it's set to the input area number)
# 
# IF we're NOT in a screen field (scr_fld is empty)
# then show help for the program (or screen):
#   hlp_mod: get_scrlib("module")
#  hlp_prog: scr_id
#            (or program name portion of progid if default screen)
#   hlp_num: 1
# 
#
    define
        tabname char(18),    # current field's table name (or formonly)
        colname char(18),    # current field's column name
        hlp_mod char(18),    # module to pass to helper (tablename)
        hlp_prog char(18),   # program to pass to helper (columname)
        hlp_num smallint,    # help number to pass
        len smallint,        # length of scr_fld
        n smallint           # generic number

    # Trap fatal errors
    whenever error call error_handler

    # Default the help num to 1
    let hlp_num = 1

    # Determine if we're in a screen field or not:
    if length(scr_fld) > 0
    then
        # We're in a screen field
        let hlp_prog = set_fldtab(scr_fld)
        let hlp_mod = get_scrlib("fld_tab")
        # Build the hlp_mod if "formonly" field
        if downshift(hlp_mod) = "formonly"
        then
            if scr_id = "default"
            then
                # "module.program"
                let hlp_mod = progid
            else
                # "module.scr_id"
                let hlp_mod = get_scrlib("module") clipped,
                  ".", scr_id clipped
            end if
            if input_num > 0 then let hlp_num = input_num end if
        end if
    else
        # We're not in a screen field.
        # Parse progid to get the module, program
        call get_mod() returning hlp_mod, hlp_prog
        if scr_id != "default"
          then let hlp_prog = scr_id
        end if
    end if

    # Pass control to helper
    call lib_help(hlp_mod,hlp_prog,hlp_num)

end function
# scr_help()
