######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/scr_reg.4gl  1.3  Delta: 1/8/91
######################################################################

globals "globals.4gl"

define
    arr_mesgs array[1] of record    # Message text
        mssg_text char(132)
    end record,
    mssg_prep char(1)                  # Y/null Message cursor prepared?

######################################################################
function scr_register(scrname, scrtype)
######################################################################
# This function registers the screen.  It remembers all of the screen
# level variables (global and library static), then sets the new
# screen level variables and calls the appropriate flow control manager
# for the specified screen type.
# Upon return from the flow control manager, scr_register restores
# all of the screen level variables to their state before the call.
#
    define
        scrname char(7),    # Screen name to register
        scrtype char(20),   # Screen type to register
        o_scrname char(7),  # Previous screen name
        o_scrtype char(20), # Previous screen type
        o_scr_fld char(40), # Previous screen field
        o_nxt_fld char(40), # Previous next field
        o_scrfunc char(20), # Previous screen function
        o_menuitem char(10), # Previous menu item
        o_numrows smallint, # Previous screen num_rows
        o_numcols smallint, # Previous screen num_cols
        o_scrtab char(18),  # Previous screen scr_tab
        o_scrkey char(30),  # Previous screen scr_key
        o_inputnum smallint,# Previous input section within screen
        o_curs_pos     integer,    # current cursor position
        o_curs_count   integer,    # number of elements in cursor
        o_curs_rowid   integer,    # rowid of current element
        o_p_cur smallint,   # Previous input array element
        o_s_cur smallint,   # Previous screen array element
        o_module char(8),        # old module
        o_sql_filter char(512),  # old sql_filter
        o_sql_order char(512),   # old sql_order
        o_prevdata char(80),# Previous data before field entry
        o_thisdata char(80),# Previous data after field entry
        o_changed smallint  # Previous has the field data changed?

    # Trap fatal errors
    whenever error call error_handler

    if mssg_prep is null then
       let arr_mesgs[1].mssg_text = fg_message("lib_scr","screg",1)
#1: "Need to add screen to switchbox() function"
       let mssg_prep = "Y"
    end if

    # Remember old values
    let o_scrname = scr_id
    let o_module = get_scrlib("module")
    let o_sql_filter = sql_filter
    let o_sql_order  = sql_order
    let o_scrtype = get_scrlib("scr_type")
    let o_menuitem = menu_item
    let o_scr_fld = scr_fld
    let o_nxt_fld = nxt_fld
    let o_scrfunc = scr_funct
    let o_numrows = get_scrlib("num_rows")
    let o_numcols = get_scrlib("num_cols")
    let o_scrtab = get_scrlib("scr_tab")
    let o_scrkey = get_scrlib("scr_key")
    let o_inputnum = input_num
    let o_curs_pos = get_scrlib("curs_pos")
    let o_curs_count = get_scrlib("curs_cnt")
    let o_curs_rowid = get_scrlib("curs_rowid")
    let o_p_cur = p_cur
    let o_s_cur = s_cur
    let o_prevdata = prev_data
    let o_thisdata = this_data
    let o_changed = data_changed

    # Set new data
    let scr_id = scrname                     # set the new scrname
    call put_scrlib("scr_type",scrtype)      # set the new scrtype
    let menu_item = "init"                   # set the new menu_item
    let scr_fld = ""                         # set the new scr_fld
    let nxt_fld = ""                         # set the new nxt_fld
    let int_flag = 0                         # clear any old int_flag

    # Initialize the local screen
    call switchbox("init")
    if scratch = "no screen"
    then
        # Screen not found in switchbox
        call lib_error("lib_scr","swbox",1,arr_mesgs[1].mssg_text) 
    else
        # Call the appropriate flow control manager
        case
          when scrtype = "zoom" call fcm_zoom()
          when scrtype = "custom" call switchbox("flow")
        end case

        # Close the screen, and return any elements in scratch
        # (don't call anything from here on that modifies scratch)
        call switchbox("close")
    end if

    # Reset the interrupt when returning from a screen
    let int_flag = 0

    # Restore the old values upon return to the previous screen
    call put_scrlib("scr_type",o_scrtype)
    call put_scrlib("num_rows",o_numrows)
    call put_scrlib("num_cols",o_numcols)
    call put_scrlib("scr_tab",o_scrtab)
    call put_scrlib("scr_key",o_scrkey)
    call put_scrlib("curs_pos", o_curs_pos)
    call put_scrlib("curs_count", o_curs_count)
    call put_scrlib("curs_rowid", o_curs_rowid)
    call put_scrlib("module",o_module)
    let sql_filter = o_sql_filter
    let sql_order = o_sql_order
    let scr_fld = o_scr_fld
    let nxt_fld = o_nxt_fld
    let scr_funct = o_scrfunc
    let menu_item = o_menuitem
    let input_num = o_inputnum
    let p_cur = o_p_cur
    let s_cur = o_s_cur
    let prev_data = o_prevdata
    let this_data = o_thisdata
    let data_changed = o_changed
    let scr_id = o_scrname

end function
# scr_register()
